"use client";
import React, { useState, useCallback, useEffect, useRef, } from "react";
import WaveSurfer from "wavesurfer.js";
import { formatTime, playbackRateOptions } from "./lib/utils";
import { updateRangeBackground } from "./lib/utils";
import Select from "./components/Select";
import "./video-audio-player.css";
const AudioPlayer = ({ src, accentColor = "#60a5fa", customErrorMessage = "An error occurred while trying to play the audio.", autoPlay = false, muted = false, loop = false, className = "", style = {}, controls = true, width = "100%", seekTo, controlsToExclude = [], showDownloadButton = false, disableShortcuts = false, defaultPlaybackRate, defaultVolume = 1, onProgress, onSeeked, onDownloadStart, onDownloadEnd, onPlaybackRateChange, onVolumeChange, onMuteChange, onPlay, onPause, onEnded, onError, onReady, onDuration, getWaveSurferRef, getAudioElement, }) => {
    const waveformRef = useRef(null);
    const [isPlaying, setIsPlaying] = useState(false);
    const [isMuted, setIsMuted] = useState(false);
    const [volume, setVolume] = useState(defaultVolume);
    const [currentTime, setCurrentTime] = useState(0);
    const [duration, setDuration] = useState(0);
    const [audioError, setAudioError] = useState(false);
    const waveSurfer = useRef(null);
    const [playbackRate, setPlaybackRate] = useState(1);
    const [containerWidth, setContainerWidth] = useState(0);
    const audioContainerRef = useRef(null);
    const volumeInputRef = useRef(null);
    const [isDownloading, setIsDownloading] = useState(false);
    const [reload, setReload] = useState(false);
    const [reverseCurrentTime, setReverseCurrentTime] = useState(false);
    useEffect(() => {
        if (waveSurfer.current) {
            setIsMuted(muted);
            setVolume(muted ? 0 : defaultVolume);
        }
    }, [muted, waveSurfer, defaultVolume]);
    useEffect(() => {
        setTimeout(() => {
            if (waveSurfer.current && defaultPlaybackRate && duration) {
                const newPlaybackRate = Math.min(Math.max(Number(defaultPlaybackRate) || 1, 0.0625), 16);
                setPlaybackRate(newPlaybackRate);
                waveSurfer.current.setPlaybackRate(newPlaybackRate);
                if (onPlaybackRateChange)
                    onPlaybackRateChange(newPlaybackRate);
            }
        }, 500);
    }, [defaultPlaybackRate, duration, onPlaybackRateChange]);
    useEffect(() => {
        if (waveSurfer.current) {
            const volumeInput = volumeInputRef.current;
            updateRangeBackground(volumeInput);
            const newVolume = Math.min(Math.max(volume || 0, 0), 1);
            waveSurfer.current.setVolume(newVolume);
            if (onVolumeChange)
                onVolumeChange(newVolume);
            if (newVolume === 0) {
                setIsMuted(true);
                if (onMuteChange)
                    onMuteChange(true);
            }
            else {
                setIsMuted(false);
                if (onMuteChange)
                    onMuteChange(false);
            }
        }
    }, [volume, onVolumeChange, onMuteChange]);
    useEffect(() => {
        const audioContainer = audioContainerRef.current;
        const handleResize = () => {
            if (audioContainer) {
                setContainerWidth(audioContainer.offsetWidth);
            }
        };
        const resizeObserver = new ResizeObserver(handleResize);
        if (audioContainer) {
            resizeObserver.observe(audioContainer);
        }
        handleResize();
        return () => {
            if (audioContainer) {
                resizeObserver.unobserve(audioContainer);
            }
        };
    }, []);
    const togglePlay = useCallback(() => {
        if (waveSurfer.current) {
            waveSurfer.current.playPause();
            setIsPlaying(!isPlaying);
        }
    }, [isPlaying]);
    const toggleMute = useCallback(() => {
        if (waveSurfer.current) {
            waveSurfer.current.setMuted(!isMuted);
            setIsMuted(!isMuted);
            setVolume(volume === 0 ? 1 : volume);
            updateRangeBackground(volumeInputRef.current, !isMuted ? 0 : volume, 1);
            if (onMuteChange)
                onMuteChange(!isMuted);
        }
    }, [isMuted, volume, onMuteChange]);
    useEffect(() => {
        if (disableShortcuts)
            return;
        const handleKeyDown = (e) => {
            const target = e.target;
            if (target.tagName === "INPUT" ||
                target.tagName === "TEXTAREA" ||
                target.tagName === "SELECT" ||
                target.tagName === "BUTTON" ||
                target.isContentEditable) {
                return;
            }
            if (waveSurfer.current) {
                switch (e.key) {
                    case "ArrowRight":
                        e.preventDefault();
                        waveSurfer.current.setTime(waveSurfer.current.getCurrentTime() + 10);
                        break;
                    case "ArrowLeft":
                        e.preventDefault();
                        waveSurfer.current.setTime(waveSurfer.current.getCurrentTime() - 10);
                        break;
                    case "ArrowUp":
                        e.preventDefault();
                        waveSurfer.current.setVolume(Math.min(volume + 0.1, 1));
                        setVolume((prevVolume) => {
                            if (prevVolume === 0) {
                                setIsMuted(false);
                            }
                            return Math.min(prevVolume + 0.1, 1);
                        });
                        break;
                    case "ArrowDown":
                        e.preventDefault();
                        waveSurfer.current.setVolume(Math.max(volume - 0.1, 0));
                        setVolume((prevVolume) => {
                            const newVolume = Math.max(prevVolume - 0.1, 0);
                            if (newVolume === 0) {
                                setIsMuted(true);
                            }
                            return newVolume;
                        });
                        break;
                    case " ":
                        e.preventDefault();
                        waveSurfer.current.playPause();
                        break;
                    case "m":
                    case "M":
                        e.preventDefault();
                        toggleMute();
                        break;
                    case "s":
                    case "S": {
                        e.preventDefault();
                        const speedRate = playbackRate === 1
                            ? 1.25
                            : playbackRate === 1.25
                                ? 1.5
                                : playbackRate === 1.5
                                    ? 1.75
                                    : playbackRate === 1.75
                                        ? 2
                                        : playbackRate === 2
                                            ? 0.25
                                            : playbackRate === 0.25
                                                ? 0.5
                                                : playbackRate === 0.5
                                                    ? 0.75
                                                    : 1;
                        setPlaybackRate(speedRate);
                        waveSurfer.current.setPlaybackRate(speedRate);
                        if (onPlaybackRateChange)
                            onPlaybackRateChange(speedRate);
                        break;
                    }
                    default:
                        break;
                }
            }
        };
        window.addEventListener("keydown", handleKeyDown);
        return () => {
            window.removeEventListener("keydown", handleKeyDown);
        };
    }, [
        isMuted,
        volume,
        playbackRate,
        disableShortcuts,
        toggleMute,
        onPlaybackRateChange,
    ]);
    const reloadAudio = () => {
        setAudioError(false);
        setTimeout(() => {
            setReload(!reload);
            updateRangeBackground(volumeInputRef.current, volume, 1);
        }, 100);
    };
    useEffect(() => {
        const handleError = () => {
            setAudioError(true);
            if (onError) {
                onError();
            }
        };
        const handleSeeked = () => {
            const currentTime = waveSurfer.current?.getCurrentTime() || 0;
            setCurrentTime(currentTime);
            if (onSeeked) {
                onSeeked(currentTime);
            }
        };
        const handlePlay = () => {
            setIsPlaying(true);
            if (onPlay) {
                onPlay();
            }
        };
        const handlePause = () => {
            setIsPlaying(false);
            if (onPause) {
                onPause();
            }
        };
        const handleEnded = () => {
            setIsPlaying(false);
            setCurrentTime(0);
            waveSurfer.current?.stop();
            if (onEnded) {
                onEnded();
            }
        };
        const handleReady = () => {
            const duration = waveSurfer.current?.getDuration() || 0;
            setDuration(duration);
            if (onReady) {
                onReady();
            }
            if (onDuration) {
                onDuration(duration);
            }
        };
        const handleClick = () => {
            waveSurfer.current?.playPause();
        };
        const handleVolumeChange = () => {
            const newVolume = waveSurfer.current?.getVolume() || 0;
            setVolume(newVolume);
            if (onVolumeChange) {
                onVolumeChange(newVolume);
            }
        };
        if (waveformRef.current) {
            waveSurfer.current = WaveSurfer.create({
                container: waveformRef.current,
                waveColor: "#9ca3af",
                progressColor: accentColor,
                cursorColor: "#000000",
                height: 50,
                url: src,
                dragToSeek: true,
                cursorWidth: 2,
                normalize: true,
                barWidth: 2,
                barGap: 1,
            });
            setCurrentTime(0);
            waveSurfer.current.on("ready", handleReady);
            waveSurfer.current.on("finish", handleEnded);
            waveSurfer.current.on("play", handlePlay);
            waveSurfer.current.on("pause", handlePause);
            waveSurfer.current.on("error", handleError);
            waveSurfer.current.getMediaElement().loop = loop;
            waveSurfer.current.on("seeking", handleSeeked);
            waveSurfer.current.getMediaElement().onvolumechange = handleVolumeChange;
            if (autoPlay) {
                waveSurfer.current.getMediaElement().autoplay = true;
            }
            if (!controls) {
                waveSurfer.current.on("click", handleClick);
            }
            if (seekTo) {
                waveSurfer.current.seekTo(seekTo);
            }
        }
        return () => {
            if (waveSurfer.current) {
                waveSurfer.current.destroy();
                waveSurfer.current.un("ready", handleReady);
                waveSurfer.current.un("finish", handleEnded);
                waveSurfer.current.un("play", handlePlay);
                waveSurfer.current.un("pause", handlePause);
                waveSurfer.current.un("seeking", handleSeeked);
                waveSurfer.current.un("error", handleError);
                if (!controls) {
                    waveSurfer.current.un("click", handleClick);
                }
                waveSurfer.current.getMediaElement().onvolumechange = null;
                waveSurfer.current.getMediaElement().loop = false;
                waveSurfer.current.getMediaElement().autoplay = false;
            }
        };
    }, [
        src,
        accentColor,
        loop,
        autoPlay,
        onReady,
        onDuration,
        onEnded,
        onError,
        onPlay,
        onPause,
        onSeeked,
        duration,
        controls,
        seekTo,
        reload,
        onVolumeChange,
    ]);
    const handleSpeedChange = (speed) => {
        setPlaybackRate(speed);
        if (waveSurfer.current) {
            waveSurfer.current.setPlaybackRate(speed);
            if (onPlaybackRateChange)
                onPlaybackRateChange(speed);
        }
    };
    useEffect(() => {
        const handleProgress = () => {
            const currentTime = waveSurfer.current?.getCurrentTime() || 0;
            setCurrentTime(currentTime);
            if (onProgress) {
                onProgress(currentTime, duration);
            }
        };
        waveSurfer.current?.on("audioprocess", handleProgress);
        return () => {
            if (waveSurfer.current) {
                waveSurfer.current.un("audioprocess", handleProgress);
            }
        };
    }, [onProgress, duration]);
    useEffect(() => {
        if (getWaveSurferRef && waveSurfer.current) {
            getWaveSurferRef(waveSurfer.current);
        }
    }, [getWaveSurferRef, waveSurfer]);
    useEffect(() => {
        if (getAudioElement && waveSurfer.current) {
            getAudioElement(waveSurfer.current.getMediaElement());
        }
    }, [getAudioElement, waveSurfer]);
    const handleVolumeChange = (e) => {
        const newVolume = parseFloat(e.target.value);
        setVolume(newVolume);
    };
    const handleDownloadClick = async () => {
        if (isDownloading)
            return;
        try {
            if (onDownloadStart)
                onDownloadStart();
            setIsDownloading(true);
            const link = document.createElement("a");
            const response = await fetch(src);
            const blob = await response.blob();
            link.href = URL.createObjectURL(blob);
            link.download =
                Math.random().toString(36).substring(2, 9) +
                    "." +
                    src.split(".").pop() || ".mp3";
            link.click();
            link.remove();
            setIsDownloading(false);
            if (onDownloadEnd)
                onDownloadEnd(true);
        }
        catch {
            setIsDownloading(false);
            if (onDownloadEnd)
                onDownloadEnd(false);
        }
    };
    return (React.createElement("div", { className: `audio-player-wrapper  ${className}`, ref: audioContainerRef, style: {
            ...style,
            "--accent-color": accentColor,
            width,
            minWidth: "90px",
        } }, audioError ? (React.createElement("div", { className: "error-message" },
        React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
            React.createElement("path", { d: "m21.73 18-8-14a2 2 0 0 0-3.48 0l-8 14A2 2 0 0 0 4 21h16a2 2 0 0 0 1.73-3" }),
            React.createElement("path", { d: "M12 9v4" }),
            React.createElement("path", { d: "M12 17h.01" })),
        React.createElement("span", null,
            React.createElement("strong", null, "Error:"),
            " ",
            customErrorMessage),
        React.createElement("button", { onClick: reloadAudio }, "Reload"))) : (React.createElement("div", { className: "controls" },
        controls && (React.createElement(React.Fragment, null,
            !controlsToExclude.includes("playPause") && (React.createElement("button", { onClick: togglePlay, className: "accent-color-hover play-pause-button", "aria-label": isPlaying ? "Pause" : "Play" }, !isPlaying ? (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                React.createElement("polygon", { points: "6 3 20 12 6 21 6 3" }))) : (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                React.createElement("rect", { x: "14", y: "4", width: "4", height: "16", rx: "1" }),
                React.createElement("rect", { x: "6", y: "4", width: "4", height: "16", rx: "1" }))))),
            React.createElement("button", { className: "current-time-duration accent-color-hover", onClick: () => {
                    setReverseCurrentTime(!reverseCurrentTime);
                }, "aria-label": "Current time and duration" },
                !controlsToExclude.includes("current-time") && (React.createElement("span", { className: `${containerWidth < 140
                        ? "hide-control"
                        : "show-control-inline-flex"}` }, reverseCurrentTime
                    ? formatTime(currentTime - duration)
                    : formatTime(currentTime))),
                !controlsToExclude.includes("duration") &&
                    !controlsToExclude.includes("current-time") && (React.createElement("span", { className: `${containerWidth < 400
                        ? "hide-control"
                        : "show-control-inline-flex"}` }, "/")),
                !controlsToExclude.includes("duration") && (React.createElement("span", { className: `${containerWidth < 400
                        ? "hide-control"
                        : "show-control-inline-flex"}` }, formatTime(duration)))))),
        React.createElement("div", { ref: waveformRef, className: "waveform", style: { width: "100%", height: "100%", cursor: "pointer" }, "aria-label": "Audio waveform" }),
        controls && (React.createElement(React.Fragment, null,
            !controlsToExclude.includes("playbackRate") && (React.createElement("div", { className: `control-relative ${containerWidth < 210
                    ? "hide-control"
                    : "show-control-inline-flex"}` },
                React.createElement(Select, { items: [
                        ...playbackRateOptions.map((rate) => ({
                            value: rate,
                            label: `${rate}x`,
                        })),
                        ...(defaultPlaybackRate &&
                            !playbackRateOptions.includes(defaultPlaybackRate)
                            ? [
                                {
                                    value: defaultPlaybackRate,
                                    label: `${defaultPlaybackRate}x`,
                                },
                            ]
                            : []),
                    ].sort((a, b) => parseFloat(a.label) - parseFloat(b.label)), value: playbackRate, ariaLabel: "Playback speed", defaultLabel: `${playbackRate}x`, onClick: (value) => {
                        const newPlaybackRate = Math.min(Math.max(Number(value) || 1, 0.0625), 16);
                        handleSpeedChange(newPlaybackRate);
                    }, key: playbackRate }))),
            !controlsToExclude.includes("mute") && (React.createElement("button", { onClick: toggleMute, className: `mute-button accent-color-hover ${containerWidth < 170 ? "hide-control" : "show-control"}`, "aria-label": isMuted || muted ? "Unmute" : "Mute" }, isMuted ? (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                React.createElement("path", { d: "M11 4.702a.705.705 0 0 0-1.203-.498L6.413 7.587A1.4 1.4 0 0 1 5.416 8H3a1 1 0 0 0-1 1v6a1 1 0 0 0 1 1h2.416a1.4 1.4 0 0 1 .997.413l3.383 3.384A.705.705 0 0 0 11 19.298z" }),
                React.createElement("line", { x1: "22", x2: "16", y1: "9", y2: "15" }),
                React.createElement("line", { x1: "16", x2: "22", y1: "9", y2: "15" }))) : (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "24", height: "24", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                React.createElement("path", { d: "M11 4.702a.705.705 0 0 0-1.203-.498L6.413 7.587A1.4 1.4 0 0 1 5.416 8H3a1 1 0 0 0-1 1v6a1 1 0 0 0 1 1h2.416a1.4 1.4 0 0 1 .997.413l3.383 3.384A.705.705 0 0 0 11 19.298z" }),
                React.createElement("path", { d: "M16 9a5 5 0 0 1 0 6" }),
                React.createElement("path", { d: "M19.364 18.364a9 9 0 0 0 0-12.728" }))))),
            !controlsToExclude.includes("volume") && (React.createElement("input", { type: "range", min: "0", max: "1", step: "any", value: isMuted ? 0 : volume, onChange: handleVolumeChange, ref: volumeInputRef, className: `volume-slider accent-color-input ${containerWidth < 400 ? "hide-control" : "show-control"}`, "aria-label": "Volume control" })),
            showDownloadButton && (React.createElement("div", { className: "download-button-wrapper" },
                React.createElement("button", { onClick: handleDownloadClick, className: "download-button accent-color", "aria-label": "Download video" }, !isDownloading ? (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "18", height: "18", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                    React.createElement("path", { d: "M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" }),
                    React.createElement("polyline", { points: "7 10 12 15 17 10" }),
                    React.createElement("line", { x1: "12", x2: "12", y1: "15", y2: "3" }))) : (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "18", height: "18", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round", className: "downloading" },
                    React.createElement("path", { d: "M3 12a9 9 0 0 1 9-9 9.75 9.75 0 0 1 6.74 2.74L21 8" }),
                    React.createElement("path", { d: "M21 3v5h-5" }),
                    React.createElement("path", { d: "M21 12a9 9 0 0 1-9 9 9.75 9.75 0 0 1-6.74-2.74L3 16" }),
                    React.createElement("path", { d: "M8 16H3v5" })))))))),
        !controls && showDownloadButton && (React.createElement("div", { className: "download-button-wrapper" },
            React.createElement("button", { disabled: isDownloading, onClick: handleDownloadClick, className: "download-button accent-color", "aria-label": "Download video" }, !isDownloading ? (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "18", height: "18", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round" },
                React.createElement("path", { d: "M21 15v4a2 2 0 0 1-2 2H5a2 2 0 0 1-2-2v-4" }),
                React.createElement("polyline", { points: "7 10 12 15 17 10" }),
                React.createElement("line", { x1: "12", x2: "12", y1: "15", y2: "3" }))) : (React.createElement("svg", { xmlns: "http://www.w3.org/2000/svg", width: "18", height: "18", viewBox: "0 0 24 24", fill: "none", stroke: "currentColor", strokeWidth: "2", strokeLinecap: "round", strokeLinejoin: "round", className: "downloading" },
                React.createElement("path", { d: "M3 12a9 9 0 0 1 9-9 9.75 9.75 0 0 1 6.74 2.74L21 8" }),
                React.createElement("path", { d: "M21 3v5h-5" }),
                React.createElement("path", { d: "M21 12a9 9 0 0 1-9 9 9.75 9.75 0 0 1-6.74-2.74L3 16" }),
                React.createElement("path", { d: "M8 16H3v5" }))))))))));
};
export default AudioPlayer;
